Stan Math Library: stan/math/prim/scal/prob/gamma_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

gamma_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_GAMMA_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_GAMMA_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
10 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
11 #include <stan/math/prim/scal/err/check_not_nan.hpp>
12 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/fun/multiply_log.hpp>
15 #include <stan/math/prim/scal/fun/value_of.hpp>
16 #include <stan/math/prim/scal/fun/gamma_p.hpp>
17 #include <stan/math/prim/scal/fun/digamma.hpp>
18 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
19 #include <stan/math/prim/scal/meta/include_summand.hpp>
20 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
21 #include <boost/random/gamma_distribution.hpp>
22 #include <boost/random/variate_generator.hpp>
23 #include <cmath>
24
25 namespace stan {
26
27 namespace math {
28
51 template <bool propto,
52 typename T_y, typename T_shape, typename T_inv_scale>
53 typename return_type<T_y, T_shape, T_inv_scale>::type
54 gamma_log(const T_y& y, const T_shape& alpha, const T_inv_scale& beta) {
55 static const char* function("stan::math::gamma_log");
56 typedef typename stan::partials_return_type<T_y, T_shape,
57 T_inv_scale>::type
58 T_partials_return;
59
60 using stan::is_constant_struct;
61 using stan::math::check_not_nan;
62 using stan::math::check_positive_finite;
63 using stan::math::check_nonnegative;
64 using stan::math::check_consistent_sizes;
65 using stan::math::value_of;
66
67 // check if any vectors are zero length
68 if (!(stan::length(y)
69 && stan::length(alpha)
70 && stan::length(beta)))
71 return 0.0;
72
73 // set up return value accumulator
74 T_partials_return logp(0.0);
75
76 // validate args (here done over var, which should be OK)
77 check_not_nan(function, "Random variable", y);
78 check_positive_finite(function, "Shape parameter", alpha);
79 check_positive_finite(function, "Inverse scale parameter", beta);
80 check_consistent_sizes(function,
81 "Random variable", y,
82 "Shape parameter", alpha,
83 "Inverse scale parameter", beta);
84
85 // check if no variables are involved and prop-to
86 if (!include_summand<propto, T_y, T_shape, T_inv_scale>::value)
87 return 0.0;
88
89 // set up template expressions wrapping scalars into vector views
90 VectorView<const T_y> y_vec(y);
91 VectorView<const T_shape> alpha_vec(alpha);
92 VectorView<const T_inv_scale> beta_vec(beta);
93
94 for (size_t n = 0; n < length(y); n++) {
95 const T_partials_return y_dbl = value_of(y_vec[n]);
96 if (y_dbl < 0)
97 return LOG_ZERO;
98 }
99
100 size_t N = max_size(y, alpha, beta);
101 OperandsAndPartials<T_y, T_shape, T_inv_scale>
102 operands_and_partials(y, alpha, beta);
103
104 using boost::math::lgamma;
105 using stan::math::multiply_log;
106 using boost::math::digamma;
107 using std::log;
108
109 VectorBuilder<include_summand<propto, T_y, T_shape>::value,
110 T_partials_return, T_y> log_y(length(y));
111 if (include_summand<propto, T_y, T_shape>::value) {
112 for (size_t n = 0; n < length(y); n++) {
113 if (value_of(y_vec[n]) > 0)
114 log_y[n] = log(value_of(y_vec[n]));
115 }
116 }
117
118 VectorBuilder<include_summand<propto, T_shape>::value,
119 T_partials_return, T_shape> lgamma_alpha(length(alpha));
120 VectorBuilder<!is_constant_struct<T_shape>::value,
121 T_partials_return, T_shape> digamma_alpha(length(alpha));
122 for (size_t n = 0; n < length(alpha); n++) {
123 if (include_summand<propto, T_shape>::value)
124 lgamma_alpha[n] = lgamma(value_of(alpha_vec[n]));
125 if (!is_constant_struct<T_shape>::value)
126 digamma_alpha[n] = digamma(value_of(alpha_vec[n]));
127 }
128
129 VectorBuilder<include_summand<propto, T_shape, T_inv_scale>::value,
130 T_partials_return, T_inv_scale> log_beta(length(beta));
131 if (include_summand<propto, T_shape, T_inv_scale>::value) {
132 for (size_t n = 0; n < length(beta); n++)
133 log_beta[n] = log(value_of(beta_vec[n]));
134 }
135
136 for (size_t n = 0; n < N; n++) {
137 // pull out values of arguments
138 const T_partials_return y_dbl = value_of(y_vec[n]);
139 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
140 const T_partials_return beta_dbl = value_of(beta_vec[n]);
141
142 if (include_summand<propto, T_shape>::value)
143 logp -= lgamma_alpha[n];
144 if (include_summand<propto, T_shape, T_inv_scale>::value)
145 logp += alpha_dbl * log_beta[n];
146 if (include_summand<propto, T_y, T_shape>::value)
147 logp += (alpha_dbl-1.0) * log_y[n];
148 if (include_summand<propto, T_y, T_inv_scale>::value)
149 logp -= beta_dbl * y_dbl;
150
151 // gradients
152 if (!is_constant_struct<T_y>::value)
153 operands_and_partials.d_x1[n] += (alpha_dbl-1)/y_dbl - beta_dbl;
154 if (!is_constant_struct<T_shape>::value)
155 operands_and_partials.d_x2[n] += -digamma_alpha[n] + log_beta[n]
156 + log_y[n];
157 if (!is_constant_struct<T_inv_scale>::value)
158 operands_and_partials.d_x3[n] += alpha_dbl / beta_dbl - y_dbl;
159 }
160 return operands_and_partials.value(logp);
161 }
162
163 template <typename T_y, typename T_shape, typename T_inv_scale>
164 inline
165 typename return_type<T_y, T_shape, T_inv_scale>::type
166 gamma_log(const T_y& y, const T_shape& alpha, const T_inv_scale& beta) {
167 return gamma_log<false>(y, alpha, beta);
168 }
169 }
170 }
171
172 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::partials_return_type
Definition: partials_return_type.hpp:16
check_less_or_equal.hpp
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
gamma_p.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::gamma_log
return_type< T_y, T_shape, T_inv_scale >::type gamma_log(const T_y &y, const T_shape &alpha, const T_inv_scale &beta)
The log of a gamma density for y with the specified shape and inverse scale parameters.
Definition: gamma_log.hpp:54
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.